//BYTE may cause problem
#ifndef _COLOR_H
#define _COLOR_H

#include <limits.h>
#include "general.h"

static const int C_RedWeight = 54;
static const int C_GreenWeight = 183;
static const int C_BlueWeight = 18;

// Grayscale color shift value
static const int CO_MONO_SHIFT = 8;

// Integer color multiplier
static const float CO_INT_MULT = (float) SHRT_MAX;

// Integer color shift value
static const int CO_INT_SHIFT = 7;

class Spectra   // Average spectral radiant exitance
{
  private:
	 float red_band;
	 float green_band;
	 float blue_band;

  public:
	 float GetBlueBand() { return blue_band; }
	 float GetGreenBand() { return green_band; }
	 float GetRedBand() { return red_band; }
	 void Limit()
	 {
		red_band = max((float) 0.0, red_band);
		green_band = max((float) 0.0, green_band);
		blue_band = max((float) 0.0, blue_band);
	 }
	 void Reset()
	 { red_band = green_band = blue_band = (float) 0.0; }
	 void SetBlueBand( float b ) { blue_band = b; }
	 void SetGreenBand( float g ) { green_band = g; }
	 void SetRedBand( float r ) { red_band = r; }

	 Spectra &Add( Spectra &a )  // Add color
	 {
		red_band += a.red_band;
		green_band += a.green_band;
		blue_band += a.blue_band;

		return *this;
	 }

	 Spectra &Subtract( Spectra &a )     // Subtract color
	 {
		red_band -= a.red_band;
		green_band -= a.green_band;
		blue_band -= a.blue_band;

		return *this;
	 }


	 Spectra &Mult( Spectra &m )    // Multiply color
	 {
		red_band *= m.red_band;
		green_band *= m.green_band;
		blue_band *= m.blue_band;

		return *this;
	 }

	 // Multiply colors
	 friend Spectra Mult( Spectra &s1, Spectra &s2 )
	 {
		Spectra temp;     // Temporary spectrum

		temp.red_band = s1.red_band * s2.red_band;
		temp.green_band = s1.green_band * s2.green_band;
		temp.blue_band = s1.blue_band * s2.blue_band;

		return temp;
	 }

	 // Blend colors
	 friend Spectra Blend( Spectra &s1, Spectra &s2, double
			alpha )
	 {
		Spectra temp;     // Temporary spectrum

		// Linear interpolation
		temp.red_band = s1.red_band + (s2.red_band -
			 s1.red_band) * (float) alpha;
		temp.green_band = s1.green_band + (s2.green_band -
			 s1.green_band) * (float) alpha;
		temp.blue_band = s1.blue_band + (s2.blue_band -
			 s1.blue_band) * (float) alpha;

		return temp;
	 }

	 double GetMaxColor()        // Get maximum color
	 {
		float maximum = (float) 0.0;

		maximum = max(maximum, red_band);
		maximum = max(maximum, green_band);
		maximum = max(maximum, blue_band);

		return (double) maximum;
	 }

	 void Scale( double value )  // Scale color
	 {
		red_band *= (float) value;
		green_band *= (float) value;
		blue_band *= (float) value;
	 }
};

class ColorRGB  // 24-bit RGB color model
{
  private:
  public:
	 BYTE red;
	 BYTE green;
	 BYTE blue;

	 BYTE GetBlue() { return blue; }
	 BYTE GetGreen() { return green; }
	 BYTE GetRed() { return red; }
	 void SetBlue( BYTE b ) { blue = b; }
	 void SetGreen( BYTE g ) { green = g; }
	 void SetRed( BYTE r ) { red = r; }

	 // Set 24-bit RGB color
	 void SetColor( Spectra &c )
	 {
		red = (BYTE) (c.GetRedBand() * (float) UCHAR_MAX);
		green = (BYTE) (c.GetGreenBand() * (float) UCHAR_MAX);
		blue = (BYTE) (c.GetBlueBand() * (float) UCHAR_MAX);
	 }

	 // Change 24-bit RGB color to grayscale
	 void SetMono()
	 {
		red = green = blue = (BYTE) (((int) red * C_RedWeight
			 + (int) green * C_GreenWeight + (int) blue *
			 C_BlueWeight) >> CO_MONO_SHIFT);
	 }

	 // Change 24-bit RGB color to pseudocolor
	 void SetPseudo()
	 {
		int gsv;          // Grayscale value

		SetMono();        // Change color to grayscale
		gsv = (int) red;  // Get grayscale value

		// Convert grayscale to pseudocolor
		if (gsv < 128)
		{
		  red = (BYTE) 0;
		  green = (BYTE) (2 * gsv);
		  blue = (BYTE) (UCHAR_MAX - 2 * gsv);
		}
		else
		{
		  red = (BYTE) (2 * gsv - UCHAR_MAX);
		  green = (BYTE) (2 * UCHAR_MAX - 2 * gsv);
		  blue = (BYTE) 0;
		}
	 }
};

class IntRGB        // Integer RGB color
{
  private:
	 int red;
	 int green;
	 int blue;

  public:
	 int GetBlue() { return blue; }
	 int GetGreen() { return green; }
	 int GetRed() { return red; }
	 void AddBlue( int b ) { blue += b; }
	 void AddGreen( int g ) { green += g; }
	 void AddRed( int r ) { red += r; }
	 void SetBlue( int b ) { blue = b; }
	 void SetGreen( int g ) { green = g; }
	 void SetRed( int r ) { red = r; }

	 // Set integer RGB color
	 void SetColor( Spectra &c )
	 {
		red = (int) (c.GetRedBand() * (float)
			 CO_INT_MULT);
		green = (int) (c.GetGreenBand() * (float)
			 CO_INT_MULT);
		blue = (int) (c.GetBlueBand() * (float)
			 CO_INT_MULT);
	 }

	 // Set 24-bit RGB color
	 void SetColorRGB( ColorRGB *pc )
	 {
		pc->SetRed((BYTE) (red >> CO_INT_SHIFT));
		pc->SetGreen((BYTE) (green >> CO_INT_SHIFT));
		pc->SetBlue((BYTE) (blue >> CO_INT_SHIFT));
	 }

	 // Subtract integer RGB color
	 friend IntRGB operator-( IntRGB &a, IntRGB &b )
	 {
		IntRGB temp;      // Temporary integer RGB color

		temp.red = a.red - b.red;
		temp.green = a.green - b.green;
		temp.blue = a.blue - b.blue;

		return temp;
	 }

	 // Subtract constant from integer RGB color
	 friend IntRGB operator-( IntRGB &a, int c )
	 {
		IntRGB temp;      // Temporary integer RGB color

		temp.red = a.red - c;
		temp.green = a.green - c;
		temp.blue = a.blue - c;

		return temp;
	 }

	 // Multiply integer RGB color by constant
	 friend IntRGB operator*( IntRGB &a, int c )
	 {
		IntRGB temp;      // Temporary integer RGB color

		temp.red = a.red * c;
		temp.green = a.green * c;
		temp.blue = a.blue * c;

		return temp;
	 }
};

#endif

