#ifndef _P_RENDER_H
#define _P_RENDER_H

#include <limits.h>
#include "out_poly.h"
#include "win_bmap.h"

// Maximum reflected color band value
static const double P_MaxColorBand = ((double) 254 / (double) 255);

// Maximum Z-buffer value
static const long PR_Infinity = LONG_MAX;

struct POINT    // Raster display point
{
  int x;        // X-axis co-ordinate
  int y;        // Y-axis co-ordinate
};

struct DDA_Info     // Edge/span DDA info
{
  int x;            // x-axis value (edge only)
  long z;           // z-axis value
  IntRGB color;     // RGB color values
};

struct VertexInfo       // Vertex information
{
  POINT ss_screen;      // Supersampling screen co-ords
  POINT screen;         // Integer screen co-ordinates
  Point3 posn;          // Scaled position
  Spectra color;        // Spectral radiant exitance
};

struct ScanInfo         // Scan line intersection info
{
  int x;                // X-axis co-ordinate
  long z;               // Pseudodepth
  IntRGB color;         // RGB color
};

struct EdgeInfo         // Edge information
{
  BOOL first;           // First intersection flag
  ScanInfo isect[2];    // Scan line intersection array
};

class PolyRender        // Polygon renderer
{
  private:
  public:

    BOOL contrast_flag;         // Contrast flag
    BOOL intensity_flag;        // Intensity flag
    int ymin;                   // Minimum y-axis co-ord
    int ymax;                   // Maximum y-axis co-ord
    int ss_ymin;                // Min supersampling co-ord
    int ss_ymax;                // Max supersampling co-ord
    int width;                  // Display width
    int height;                 // Display height
    int num_vert;               // Number of vertices
    int ss_rate;                // Supersampling rate
    double contrast;            // Contrast scaling factor
    double intensity;           // Intensity scaling factor
    double color_scale;         // Color scaling factor
    long **z_buffer;            // Depth buffer pointer
    EdgeInfo *edge_list;        // Edge list pointer
    DDA_Info pi;                // DDA pixel parameters
    DDA_Info si;                // DDA delta values
    DDA_Info r;                 // DDA offset error terms
    DDA_Info inc;               // DDA increment values
    DDA_Info dec;               // DDA decrement values
    Spectra intensity_adj;      // Intensity adjustment
    VertexInfo v_info[8];       // Vertex info table
//    WinBitmap *pbmap;           // Bitmap object pointer

    // Calculate integer floor(a / b) - assumes b > 0
    int IntFloorDiv( int a, int b )
    {
      if (a >= 0)
        return a / b;
      else
        return (a / b) + ((a % b) == 0 ? 0 : - 1);
    }

    // Calculate long floor(a / b) - assumes b > 0
    long LongFloorDiv( long a, long b )
    {
      if (a >= 0)
        return a / b;
      else
        return (a / b) + ((a % b) == 0 ? 0 : - 1);
    }

    // Scale z-axis value from float to long (assumes
    // range is 0.0 to 1.0)
    long ScaleZ( double x )
    { return ((long) (x * (double) LONG_MAX)); }

    void EdgeSetup( VertexInfo *, VertexInfo * );
    void EdgeScan( VertexInfo *, VertexInfo * );
    void GetVertexInfo( OutPolygon &, double, double );
    void ScanEdges();
    void DrawEdgeList();
    void SpanSetup( ScanInfo *, ScanInfo * );
    void SpanScan( ScanInfo *, ScanInfo *, int );

    PolyRender()
    {
      contrast_flag = FALSE;
      intensity_flag = FALSE;
      contrast = 1.0;
      intensity = 0.0;
    }

    BOOL ContrastFlag() { return contrast_flag; }
    BOOL IntensityFlag() { return intensity_flag; }
    BOOL Open(int x, int y/* WinBitmap * */, double );
    double GetContrast() { return contrast; }
    double GetIntensity() { return intensity; }
    void CalcColorScale( double rmax )
    {
      if (rmax > MIN_VALUE)
        color_scale = P_MaxColorBand / rmax;
      else
        color_scale = 1.0;
    }
    void Close();
    void DisableContrast() { contrast_flag = FALSE; }
    void DisableIntensity() { intensity_flag = FALSE; }
    void EnableContrast() { contrast_flag = TRUE; }
    void EnableIntensity() { intensity_flag = TRUE; }
    void Render( OutPolygon &, int, double, double );
    void Reset();
    void SetContrast( double c ) { contrast = c; }
    void SetIntensity( double i )
    {
      intensity = i;
      intensity_adj.SetRedBand((float) i);
      intensity_adj.SetGreenBand((float) i);
      intensity_adj.SetBlueBand((float) i);
    }
};

#endif

