#include <string.h>
#include "../include/win_bmap.h"

// Open bitmap
BOOL WinBitmap::Open( int w, int h )
{
  FreeBitmap();     // Release current bitmap (if any)
  FreePalette();    // Release logical palette (if any)

  width = w;
  height = h;

  // Bitmap width must be multiple of DWORD (4 bytes) to
  // avoid segmentation arithmetic problems with __huge
  // pointers on 80x86 CPU
  pal_scan = (DWORD) BM_WidthBytes(width);
  rgb_scan = (DWORD) BM_WidthBytes(width * BM_RGB_BPP);

  // Calculate bitmap size
  pal_size = pal_scan * (DWORD) height;
  rgb_size = rgb_scan * (DWORD) height;

  if (AllocBitmap() == FALSE)   // Allocate bitmap
  {
    width = height = 0;
    pal_scan = rgb_scan = (DWORD) 0L;
    pal_size = rgb_size = (DWORD) 0L;

    return FALSE;
  }

  InitDIB();    // Initialize DIB palette

  return TRUE;
}

// Perform color quantization
BOOL WinBitmap::QuantizeColors()
{
  if (Quantize() == FALSE)      // Quantize bitmap colors
    return FALSE;

  if (AllocPalette() == FALSE)  // Create logical palette
    return FALSE;

  InitDIB();    // Reinitialize DIB palette
  
  return TRUE;
}


BOOL WinBitmap::Write( char *fname )
{
  HFILE hfile;  // File handle

  // Check for existing bitmap
  if (pbm == NULL)
    return FALSE;

  // Open the file
  if ((hfile = _lcreat(fname, 0)) == HFILE_ERROR)
    return FALSE;

  if (rgb_flag == FALSE)
  {
    SetDIBPalette();    // Initialize the DIB palette

    // Write the DIB palette
    _lwrite(hfile,(LPSTR) bm_colors, sizeof(RGBQUAD) *
        num_colors);

    // Write the palette-mapped bitmap
    WriteBitmap(hfile, pal_size);
  }
  else
  {
    // Write the RGB bitmap
    WriteBitmap(hfile, rgb_size);
  }

  fclose(hfile);       // Close the file

  return TRUE;
}

void WinBitmap::Close()     // Close bitmap
{
  FreeBitmap();     // Release bitmap
  FreePalette();    // Release logical palette

  width = height = 0;
  pal_scan = rgb_scan = (DWORD) 0L;
  pal_size = rgb_size = (DWORD) 0L;
}

// Initialize DIB data
void WinBitmap::InitDIB()
{
  // Initialize DIB file header
  bm_file.bfOffBits = (DWORD) (sizeof(BITMAPFILEHEADER)
      + sizeof(BITMAPINFOHEADER) + sizeof(RGBQUAD) *
      num_colors);
  if (rgb_flag == TRUE)
    bm_file.bfSize = (DWORD) (bm_file.bfOffBits + rgb_size);
  else
    bm_file.bfSize = (DWORD) (bm_file.bfOffBits + pal_size);

  // Initialize DIB information header
  bm_iheader.biWidth = width;
  bm_iheader.biHeight = height;
  bm_iheader.biClrUsed = num_colors;
  bm_iheader.biClrImportant = num_colors;

  if (rgb_flag == TRUE)
    bm_iheader.biBitCount = 24;
  else
    bm_iheader.biBitCount = 8;
}

// Write bitmap to file
BOOL WinBitmap::WriteBitmap( int hfile, DWORD size )
{
  DWORD remain = size;          // Bytes remaining
  BYTE __huge *pbuff = pbm;     // Buffer pointer

  // Write buffer to file in blocks
  while (remain > (DWORD) BM_BlockSize)
  {
    if (_lwrite(hfile, (LPSTR) pbuff, BM_BlockSize) !=
        BM_BlockSize)
      return FALSE;
    remain -= BM_BlockSize;
    pbuff += BM_BlockSize;
  }

  // Write last block to file
  if ((DWORD) _lwrite(hfile, (LPSTR) pbuff, (WORD) remain)
      == remain)
    return TRUE;
  else
    return FALSE;
}

void WinBitmap::FlushBitmap()   // Clear bitmap
{
  int i;                // Loop index
  BYTE __huge *prow;    // Row pointer

  // Initialize row pointer
  prow = pbm;

  for (i = 0; i < height; i++)
  {
    memset(prow, '\0', (size_t) rgb_scan);
    prow += rgb_scan;
  }
}

BOOL WinBitmap::AllocBitmap()   // Allocate bitmap
{
  // Allocate global memory for bitmap
  if ((hdib = GlobalAlloc(GMEM_MOVEABLE | GMEM_ZEROINIT,
      rgb_size)) != NULL)
  {
    // Lock bitmap memory
    pbm = (BYTE __huge *) GlobalLock(hdib);
    return TRUE;
  }
  else
    return FALSE;
}

BOOL WinBitmap::AllocPalette()  // Allocate logical palette
{
  int i;                // Loop index
  LOGPALETTE *ppal;     // Logical palette pointer

  // Allocate palette
  if ((ppal = (LOGPALETTE *) new BYTE[sizeof(LOGPALETTE)
      + num_colors * sizeof(PALETTEENTRY)]) == NULL)
    return FALSE;

  // Initialize palette information
  ppal->palVersion = 0x300;
  ppal->palNumEntries = (WORD) num_colors;

  // Copy DIB palette
  for (i = 0; i < num_colors; i++)
  {
    ppal->palPalEntry[i].peRed = palette[i].GetRed();
    ppal->palPalEntry[i].peGreen = palette[i].GetGreen();
    ppal->palPalEntry[i].peBlue = palette[i].GetBlue();
    ppal->palPalEntry[i].peFlags = (BYTE) 0;
  }

  // Create logical color palette
  hpal = CreatePalette(ppal);

  delete [] ppal;       // Release palette memory

  return TRUE;
}

void WinBitmap::FreeBitmap()    // Release bitmap
{
  if (hdib != NULL)     // Release DIB memory
  {
    GlobalUnlock(hdib);
    GlobalFree(hdib);
    pbm = NULL;
    hdib = NULL;
  }

  if (hddb != NULL)     // Release DDB memory
  {
    DeleteObject(hddb);
    hddb = NULL;
  }
}

void WinBitmap::FreePalette()   // Release logical palette
{
  if (hpal != NULL)
  {
    DeleteObject(hpal);
    hpal = NULL;
  }
}

int main(){}
